@@ -65,6 +65,11 @@ struct SSpecialCategory {
     size_t anonymousID = 0;
 };
 
+enum eGetNextLineFailure : uint8_t {
+    GETNEXTLINEFAILURE_EOF = 0,
+    GETNEXTLINEFAILURE_BACKSLASH,
+};
+
 class CConfigImpl {
   public:
     std::string path         = "";
@@ -94,4 +99,4 @@ class CConfigImpl {
     struct {
         bool noError = false;
     } currentFlags;
-};
\ No newline at end of file
+};
